/*
complainer.h 5.1.2 (2002-Nov-04-Mon)
Adam M. Costello
http://www.nicemice.net/amc/

Functions for error reporting and abnormal termination.

This is ANSI C code.

*/


#ifndef COMPLAINER_H
#define COMPLAINER_H


int warnf(const char *format, ...);

    /* Like printf, but prints to stderr and flushes it.  The format  */
    /* may be preceeded by the identifier code_ref, in which case the */
    /* format is effectively appended with " at %s line %d\n" and the */
    /* argument list is effectively appended with __FILE__, __LINE__. */


void failf(const char *format, ...);

    /* Like warnf, but then calls exit(EXIT_FAILURE). */


void abortf(const char *format, ...);

    /* Like warnf, but then calls abort(). */


extern const char * const warnf_code_ref_hook;
#define code_ref warnf_code_ref_hook, __FILE__, __LINE__


/* A particularly common failure condition: */

extern const char *out_of_memory;  /* "out of memory\n" */

#include <stdlib.h>

static
#ifdef INLINE
INLINE
#endif
void *malloc_or_fail(size_t size)
{
  void *p;

  (void) malloc_or_fail;  /* quiet function-not-used warnings */
  p = malloc(size);
  if (p == 0) failf(out_of_memory);
  return p;
}

/* If your compiler supports inline declarations, you can define     */
/* INLINE as inline (or __inline__, or whatever it takes).  Even if  */
/* INLINE is not defined, it might be possible to tell an optimizing */
/* compiler to inline functions of its own choosing.                 */


#endif /* COMPLAINER_H */
