/*
delayer.h 0.0.1 (2003-Apr-05-Sat)
Adam M. Costello  
http://www.nicemice.net/amc/

An end-to-end network delay generator.

This is ANSI C code.

*/


#ifndef DELAYER_H
#define DELAYER_H


/*** Incomplete types ***/

struct delayer;
    /* The entity that keeps the state needed to generate delays. */

struct rng;
    /* See rng.h. */


/*** Complete types ***/

/* The interior of the following structure is not part of the */
/* public interface, and is subject to change.  Callers must  */
/* not look inside.                                           */

struct delayer_endpoint {
  unsigned short region, point;
};


/*** Functions for using a delayer ***/

struct delayer *delayer_new(struct rng *);

    /* delayer_new(rng) returns a pointer to a new delayer, or a */
    /* null pointer if one cannot be created.  The delayer will  */
    /* use the given random number generator.                    */


void delayer_delete(struct delayer *);

    /* Deletes the delayer.  After it has been deleted, */
    /* it must not be passed to any functions.          */


int delayer_endpoint_alloc(struct delayer *, struct delayer_endpoint *);

    /* delayer_endpoint_alloc(delayer,&endpoint) allocates an         */
    /* available endpoint uniformly at random, writes it to endpoint, */
    /* and returns 1.  If no endpoints are available, endpoint is     */
    /* left untouched, and 0 is returned.                             */


void delayer_endpoint_free(struct delayer *, struct delayer_endpoint);

    /* Frees the given endpoint, which must have been unavailable */
    /* due to an earlier call to delayer_endpoint_alloc().        */


double delayer_delay( struct delayer *,
                      struct delayer_endpoint, struct delayer_endpoint );

    /* delayer_delay(delayer,src,dest) returns a delay from src to */
    /* dest.  If called again with the same arguments, the return  */
    /* value may be different (if the model includes randomness).  */


#endif /* DELAYER_H */
