/*
malloc 0.0.1 (2003-Aug-10-Sun)
Adam M. Costello  
http://www.nicemice.net/amc/

Implements malloc.h 0.0.x.

This is C99 code that works only with the malloc() from GNU libc 2.3 or
compatible!  It looks inside the private data structure of memory blocks
allocated by malloc().

*/


#include "malloc.h"

#undef malloc
#undef free

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>

enum { num_sizes = 1024 };

/*

I generally don't use non-constant global/static variables, because
under-the-table communication is dangerous.  But for this debugging
trick, it's expedient.

*/

static long histogram[num_sizes];


void *malloc_wrapper(size_t size)
{
  uint32_t *p;
  long i;

  p = malloc(size);
  if (p == 0) return 0;
  i = p[-1] >> 3;

  if (i < num_sizes) ++histogram[i];
  else {
    ++histogram[0];
    fprintf(stderr, "malloc_wrapper(%lu)\n", (unsigned long) size);
  }

  return p;
}


void free_wrapper(void *v)
{
  uint32_t *p = v;
  long i = p[-1] >> 3;

  if (i < num_sizes) --histogram[i];
  else {
    --histogram[0];
    fprintf(stderr, "malloc_free(): %ld\n", i << 3);
  }

  free(p);
}


void malloc_histogram(void)
{
  int i;

  for (i = 0;  i < num_sizes;  ++i) {
    fprintf( stderr, "%4d: %6ld (%3ldMB)\n",
             i, histogram[i], (i * histogram[i]) >> 17);
  }
}
