/*
rnguint.c 0.1.2 (2003-Apr-03-Thu)
Adam M. Costello
http://www.nicemice.net/amc/

Implements any one of the public rng_* functions that return unsigned
integral types.  This file is to be #included by rng.c.  Three temporary
preprocessor symbols must be defined:

rng_uinttype(rng,max) must be defined as the_real_name(rng,max).

uinttype must be defined as the unsigned integral return type (just the
type name, not surrounded by parentheses).

UINTTYPE_MAX must be defined as the maximum representable value of the
return type.

All three symbols will be undefined after this file is included.

In addition, the function rng_rand() must exist (but may be a
preprocessor symbol) and must have the same interface as the ANSI rand()
function.

This is ANSI C code.

*/


uinttype rng_uinttype(struct rng *rng, uinttype max)
{
  uinttype scale, q, maxq, maxr, r;

  /* It will be convenient to eliminate some trivial corner cases. */

  if (max == 0) return 0;
  if (max == RAND_MAX) return rng_rand(rng);

  if (max < RAND_MAX) {
    scale = ( (unsigned int) RAND_MAX
      #if RAND_MAX < UINT_MAX || RAND_MAX < UINTTYPE_MAX
        + (uinttype)1
      #endif
      ) / (max + 1);

    /* The numerator is RAND_MAX + 1 if that can be represented       */
    /* in the wider of uinttype and unsigned int, otherwise it's      */
    /* RAND_MAX.  The smaller numerator might reduce efficiency, but  */
    /* it will still give correct results.  The denominator max + 1   */
    /* cannot overflow because max < RAND_MAX and RAND_MAX fits in an */
    /* int.                                                           */

    do q = rng_rand(rng) / scale;
    while (q > max);
    return q;
  }

  /* max > RAND_MAX */

  scale = max / ((uinttype) RAND_MAX + 1) + 1;

  /* If max > RAND_MAX, then RAND_MAX can be represented as a         */
  /* uinttype, and can be incremented without overflow.  The scale is */
  /* max + 1 divided by RAND_MAX + 1 and rounded upward.              */

  maxq = max / scale;
  maxr = max % scale;

  do {
    r = rng_rand(rng);
    q = rng_uinttype(rng,maxq);
  } while (q == maxq && r > maxr);

  return q * scale + r;
}


#undef rng_uinttype
#undef uinttype
#undef UINTTYPE_MAX
