/*
rttdata.h 0.2.0 (2003-Apr-11-Fri)
Adam M. Costello  
http://www.nicemice.net/amc/

Constant data that can be used to generate random round-trip times.

Currently the sample points are represented as floats, which is
overkill, but easy to use.  A future version of this interface may
use "half" instead (unsigned short interpreted as a 16-bit IEEE-like
floating point format, with 1 sign bit, a 5-bit exponent, and 10-bit
fraction).  This would cut the memory in half, but would require a
bit of work to decode (a shift, two masks, one table lookup, and one
multiply).

This is ANSI C code.

*/


#ifndef RTTDATA_H
#define RTTDATA_H


extern const unsigned int rttdata_node_count;    /* number of nodes */
extern const unsigned int rttdata_sample_count;  /* samples per CDF */

extern const double rttdata_sample_freq;

    /* For each pair of nodes there is a cummulative probability      */
    /* distribution (CDF) approximated by sample points, where        */
    /* sample_freq is the sampling frequency along the percentile     */
    /* axis.  The reciprocal of sample_freq is the sampling period,   */
    /* which is the spacing between consecutive samples.  The samples */
    /* are spaced evenly and are symmetric around 50%.  For example,  */
    /* if sample_count is 7 and sample_freq is 5.0, then the sample   */
    /* percentiles are -10%, 10%, 30%, 50%, 70%, 90%, 110%.  If       */
    /* sample_count is 8 and sample_freq is 5.0, the percentiles      */
    /* are -20%, 0%, 20%, 40%, 60%, 80%, 100%, 120%.  Of course in    */
    /* a real CDF the percentile must be between 0% and 100%, but     */
    /* the values outside that range may be helpful or necessary      */
    /* for interpolation.  The higher layers that use this data       */
    /* might require a certain relation between sample_freq and       */
    /* sample_count.  For example, if no extrapolation is performed,  */
    /* and if the interpolation function uses four samples (two on    */
    /* each side), then sample_freq would need to be no greater       */
    /* than sample_count - 3.  The percentiles of the samples are     */
    /* typically not truly known; they are estimates.                 */

extern const float rttdata[];

    /* The first rttdata_sample_count items in the array form the    */
    /* CDF for the node pair {0,1}.  The next group form the CDF for */
    /* the pair {0,2}, then {1,2}, {0,3}, {1,3}, {2,3}, {0,4}, and   */
    /* so on.  The pairs are indeed sets, not ordered pairs, because */
    /* the samples are RTTs, not one-way delays.  The array contains */
    /* no RTTs between a node and itself (presumably that is zero).  */
    /* Within each CDF, the samples appear in order of increasing    */
    /* percentile, and have nondecreasing RTTs.                      */

extern const double rttdata_region_scale[];

    /* A table of scale factors (all in the range 0.0 to 1.0) to use  */
    /* to scale down the mesh to act as a local region.  There is one */
    /* scale factor for each node.  The intent is that each endpoint  */
    /* is a <region,point> pair, where the two numbers are distinct   */
    /* node indices.  The RTT between two endpoints in the same       */
    /* region is simply the scaled RTT between the two point-nodes.   */
    /* The RTT between two endpoints in different regions is the      */
    /* sum of the unscaled RTT between the two region-nodes and the   */
    /* scaled RTTs between each region-node and its point-node.       */


#endif /* RTTDATA_H */
