/*
simrouter.h 0.1.1 (2003-Apr-12-Sat)
Adam M. Costello  
http://www.nicemice.net/amc/

This is the backdoor interface to the router, to be used in simulations.
The public interface is router.h.

This is ANSI C code.

Generally, I don't use non-constant static data, because under-the-table
communication is dangerous.  But for the simulation, it's expedient.
The variables declared here are defined in the router module, and can
be altered to affect the behavior of the router.

*/


#ifndef SIMROUTER_H
#define SIMROUTER_H


#include "simglobals.h" /* version 0.3.x */


/* The following two variables are backdoor arguments to             */
/* router_new().  They may be altered whenever no router is running. */
/* In other words, treat alterations of these variables like calls   */
/* into the router module.  Presumably these variables will be       */
/* altered just before every call to router_new().                   */

extern sim_index_t simrouter_initial_peers_n;  /* number of initial peers */
extern sim_index_t *simrouter_initial_peers;   /* array of their indices  */

/* The next variable is initially a null pointer, but can be set */
/* (whenever no router is running) to point to a function to be  */
/* called whenever a router becomes "warmed up" (whatever that   */
/* means).  Each router becomes warmed up at most once.          */

typedef void simrouter_Warmup_Callback(
  double, struct router *, const unsigned char * );

extern simrouter_Warmup_Callback *simrouter_warmup_callback;

    /* simrouter_warmup_callback(now,router,address) is called when  */
    /* a router becomes "warmed up".  The arguments are the router's */
    /* scheduler's uptime, a pointer to the router, and the router's */
    /* communicator address.                                         */


#endif /* SIMROUTER_H */
