/*
simtangle.h 0.6.0 (2003-Jul-18-Fri)
Adam M. Costello  
http://www.nicemice.net/amc/

This is the backdoor interface to the tangle module, to be used in
simulations.  The public interface is router.h.

This is ANSI C code.

Generally, I don't use non-constant static data, because under-the-table
communication is dangerous.  But for the simulation, it's expedient.
The variables declared here are defined in the tangle (router) module,
and can be altered to affect its behavior.

*/


#ifndef SIMTANGLE_H
#define SIMTANGLE_H


/*** Incomplete types ***/

struct router;


/*** Complete types ***/

enum {
  simtangle_lookup_default = -1,
  simtangle_lookup_reduce_time = -2,
  simtangle_lookup_reduce_hops = -3,
  simtangle_lookup_uniform = -4,
  simtangle_lookup_min_time = -5
};

/* The following classes are guaranteed to be consecutive starting */
/* at zero, so that the last symbol (simtangle_msg_class_n) can be */
/* used to dimension an array with one entry for each class.       */

enum simtangle_msg_class {
  simtangle_msg_peer,
  simtangle_msg_ad,
  simtangle_msg_lookup,
  simtangle_msg_class_n
};

/* The following symbols are guaranteed to be 0 and  */
/* 1, so that an array of dimension 2 can hold both. */

enum simtangle_msg_direction {
  simtangle_sent = 0,
  simtangle_received = 1
};


/*** Functions ***/

void simtangle_set_lookup_style(struct router *, int);

    /* simtangle_set_lookup_style(router, lookup_style) sets          */
    /* the lookup style for future lookups (but does not              */
    /* affect lookups that already exist).  A lookup style is         */
    /* a policy that determines the forwarding hint for each          */
    /* query in the lookup.  Four of the lookup styles are            */
    /* trivial, using the same forwarding hint for all queries:       */
    /* simtangle_lookup_{reduce_time,reduce_hops,uniform,min_time}    */
    /* (all of which are negative values).  The fifth lookup style    */
    /* is a hybrid style parameterized by a nonnegative constant: if  */
    /* lookup_style is a nonnegative value c then the first query     */
    /* will use the min_time hint, the next c queries will use the    */
    /* reduce_time hint, and all subsequent queries will alternate    */
    /* between the reduce_hops and uniform hints.  If lookup_style is */
    /* simtangle_lookup_default (which is the initial value) then a   */
    /* default style is selected.                                     */


void simtangle_freeze(struct router *);

    /* Halts the following activities: peer discovery, local resource */
    /* advertisements, and location timeouts.  The peer state and     */
    /* location state are frozen.  Local resource advertisements      */
    /* might resume if the shipping address is set or if a new local  */
    /* resource is added.                                             */


unsigned long simtangle_get_msg_tally( struct router *,
  enum simtangle_msg_direction, enum simtangle_msg_class );

    /* simtangle_get_msg_tally(router,direction,class) returns   */
    /* the total number of messages of the given class that have */
    /* been sent or received (depending on direction).  The type */
    /* and direction must be valid.                              */


/*** Shared variables ***

The following variables are backdoor arguments to router_new().  They
may be altered whenever no router is running.  In other words, treat
alterations of these variables like calls into the router module.

*/

extern float simtangle_query_period;

    /* The average period at which queries */
    /* are sent while a lookup is active.  */


extern int simtangle_relay_bug;

    /* If nonzero when a router is created, the router will be unable */
    /* to relay queries and indirect reports (but will otherwise      */
    /* handle them correctly).  The initial value is zero.            */


#endif /* SIMTANGLE_H */
